unit ZLIBArchiveGlobals2;

interface

uses SysUtils, Classes;

resourcestring
   ArchiveFileNotFound = 'Archive file not found';
   UnableToOpenFile = 'Unable to open archive %s';
   VerifyingArchive = 'Verifying archive integrity';
   ArchiveCorrupted = 'The archive appears to be corrupted';
   ExtractingRegistry = 'Extracting registry';
   VersionNotKnown = 'The archive version is greater than the version of this component';
   RegistryReadError = 'Error reading registry';
   NoArchiveOpen = 'No archive is currently open';
   IndexBounds = 'Index out of bounds';
   FilePathError = 'Error extracting file to %s';
   ExtractingFile = 'Extracting %s';
   SkippingFile = 'Skipping %s';
   FileCRC = '%s failed CRC check';
   FileNotFound = '%s not found in archive';
   FileAlreadyExists = '%s already exists in the archive';
   UpdatingFile = 'Updating %s';
   MovingTempFile = 'Moving and renaming the temporary file';
   WritingRegistry = 'Writing archive registry';
   OverwriteFile = 'Replace %s (Yes/No/Cancel/All):';
   OverwriteFile2 = 'Replace %s';
   ArchiveDataNotFound = 'No valid archive data found';
   ArchiveNotProperlyFormed = 'This file does not appear to be a properly formed archive';
   NoDecryptionDetected = 'No decryption algorithm found';
   ZLBArchiveVersionError = 'The archive was created with a different version of ZLBArchive (v%d)';
   OlderArchiveFormat = 'The specified archive file was created by an older version of ZLBArchive.'#13#10'Please convert this archive before trying to modify/update it';
   NoPasswordGiven = 'No password given for encrypted file';
   AlreadyCurrentArchiveFormat = 'The specified archive is already marked as the current version';
   ArchiveFileAlreadyExists = 'That archive file name already exists, please choose another file name';


type
   TCRC32Table = array[0..255] of longint;

   TFileStatus = (fsCompressed, fsStored, fsCompressedEncrypted, fsStoredEncrypted);
   TFileCompressionLevel = (fcNone, fcFastest, fcDefault, fcMaximum) ;
   TInternalFlags = (ifOpening, ifExtracting, ifAdding, ifUpdating, ifWritingRegistry, ifClear, ifVerifyingArchiveCRC, ifMovingTempFile) ;

   TFileAddUpdateEvent = procedure(var Path, Name: shortstring) of object;
   TReplaceFileEvent = procedure(Filename: string; var Replace: boolean) of object;
   TFileExtractedEvent = procedure(filename: string) of object;
   TErrorLogEvent = procedure(Msg: string) of object;
   TProgressEvent = procedure(Msg: string; Percentage: longint) of object;
   TStreamDecryptEvent = procedure(StrmIn, StrmOut:TStream; Size:Cardinal; Password:string) of object;
   TStreamEncryptEvent = procedure(StrmIn, StrmOut:TStream; Password:string) of object;

   EZLBArchiveException = Exception;

   EFileCorruption = EZLBArchiveException;
   EFileNotFound = EZLBArchiveException;
   EArchiveRegistry = EZLBArchiveException;
   EGeneralArchiveException = EZLBArchiveException;
   EExtractionError = EZLBArchiveException;
   ENoArchive = EZLBArchiveException;
   EArchiveIndexBounds = EZLBArchiveException;
   EFileNotFoundInArchive = EZLBArchiveException;
   EVersionError = EZLBArchiveException;

   TZLBFileRec = record
      name: shortstring;
      path: shortstring;
      osize: longint;
      csize: longint;
      start: longint;
      CRC: longint;
      Status: TFileStatus;
      CompressionLevel: TFileCompressionLevel;
   end;

   TZLBFileRec2 = packed record
      name: shortstring;
      path: shortstring;
      osize: Int64;
      csize: Int64;
      esize: Int64;
      start: Int64;
      CRC: longint;
      Status: TFileStatus;
      CompressionLevel: TFileCompressionLevel;
   end;

   TZLBArchiveRec = record
      CRC: longint;
      FileCount: longint;
      Version: longint;
      RegistrySize: longint;
      ArchiveSize: longint;
      CompressedRegistry: Boolean;
      Reserved: array[0..11] of byte;
   end;

   TZLBArchiveRec2 = packed record
      CRC: longint;
      FileCount: cardinal;
      Version: longint;
      RegistrySize: cardinal;
      ArchiveSize: int64;
      CompressedRegistry: Boolean;
      ArchiveEncrypted : boolean;
      Reserved: array[0..9] of byte;
   end;

var
   oCRC32Table: TCRC32Table;

Function InitCRC32:TCRC32Table;
function GetCRC32(DataStream: TStream; IgnoreCount, StartPos: Int64) : longint;

implementation

Function InitCRC32:TCRC32Table;
var
   crc, poly: longint;
   i, j: longint;
begin
   poly := longint($EDB88320) ;
   for i := 0 to 255 do
   begin
      crc := i;
      for j := 8 downto 1 do
      begin
         if (crc and 1) = 1 then
            crc := (crc shr 1) xor poly
         else
            crc := crc shr 1;
      end;
      InitCRC32[i] := crc;
   end;
end;

function GetCRC32(DataStream: TStream; IgnoreCount, StartPos: Int64): longint;
var
   crc, checked, buffersize, fsize, count: longint;
   BufferArray: array[0..10239] of byte;
   originalposition: longint;
begin
   originalposition := datastream.Position;
   dataStream.Seek(StartPos, soFromBeginning);
   crc := longint($FFFFFFFF) ;
   fsize := (datastream.size-StartPos) - IgnoreCount;
   while True do
   begin
      if fsize <= 0 then break;
      if fsize >= 10240 then
         buffersize := 10240
      else
         buffersize := fsize;
      Count := DataStream.Read(BufferArray, BufferSize) ;
      checked := 0;
      while checked < Count do
      begin
         crc := ((crc shr 8) and $FFFFFF) xor oCRC32Table[(crc xor bufferArray[checked]) and $FF];
         inc(checked) ;
      end;
      dec(fsize, buffersize) ;
   end;
   result := (crc xor longint($FFFFFFFF) ) ;
   datastream.seek(originalposition, soFromBeginning) ;
end;

initialization
   oCRC32Table := InitCRC32;
end.
